from http import HTTPStatus

from marshmallow import Schema, fields
from marshmallow.validate import OneOf

from .constant import METHODS


class WhenConfigSchema(Schema):
    path = fields.String(required=True)
    method = fields.String(default='GET', validate=OneOf(METHODS))
    type = fields.String(default='application/json', validate=OneOf(['application/json']))
    header = fields.Dict()
    body = fields.Dict()
    query = fields.Dict()


class ThenConfigSchema(Schema):
    method = fields.String(default='GET', validate=OneOf(METHODS))
    type = fields.String(default='application/json', validate=OneOf(['application/json']))
    header = fields.Dict()
    body = fields.Raw()
    code = fields.Int(default=HTTPStatus.OK, validate=OneOf(list(map(int, HTTPStatus))))


class ConfigSchema(Schema):
    id = fields.String(required=True)
    description = fields.String()
    when = fields.Nested(WhenConfigSchema, required=False)
    then = fields.Nested(ThenConfigSchema, required=False)



import argparse
import os
import os.path as path

from _pyrrot.main import create_app


def _create_args():
    parser = argparse.ArgumentParser(description='Foo')
    parser.add_argument('-o', '--host', help='The host of the pyrrot. Defaults to 0.0.0.0', type=str, default='0.0.0.0')
    parser.add_argument('-p', '--port', help='The port of the pyrrot. Defaults to 1234', type=int, default=1234)
    requiredNamed = parser.add_argument_group('required named arguments')
    requiredNamed.add_argument('-c', '--conf', help='Directory path to read configurations', required=True)
    return parser.parse_args()


def _watch_changed_files(extra_dirs):
    extra_files = extra_dirs[:]
    for extra_dir in extra_dirs:
        for dirname, dirs, files in os.walk(extra_dir):
            for filename in files:
                filename = path.join(dirname, filename)
                if path.isfile(filename):
                    extra_files.append(filename)
    return extra_files


def main(developer_mode=False):
    conf = '../examples'
    port = 1234
    host = '0.0.0.0'
    extra_dirs = [conf, ]
    if not developer_mode:
        args = _create_args()
        conf = args.conf
        port = args.port
        host = args.host
        extra_dirs = [conf, ]

    extra_files = _watch_changed_files(extra_dirs)

    app = create_app(debug=True, configuration=conf)
    app.run(host=host, port=port, extra_files=extra_files)


if __name__ == "__main__":
    main(developer_mode=True)



import os

from flask import Flask

from .config import read_configs
from .rule import register_rules, register_exceptions


def create_app(debug=False, configuration=""):
    app = Flask(__name__)
    app.debug = debug
    register_rules(app)
    register_exceptions(app)
    read_configs(app, os.path.join(os.getcwd(), configuration))
    return app



# -*- coding: utf-8 -*-
from setuptools import setup


def main():
    extras_require = {}
    install_requires = [
        "PyYAML==3.12",
        "Flask==0.11.1",
        "marshmallow==2.9.1"
    ]

    setup(
        name="pyrrot",
        version='0.0.5',
        description=u"pyrrot: É um simples serviço feito em python para simular a resposta de aplicações rest.",
        url="https://github.com/akiranukamoto/pyrrot",
        license="MIT license",
        platforms=["unix", "linux", "osx", "cygwin", "win32"],
        author="Akira Nukamoto",
        author_email='akiranukamoto@gmail.com',
        entry_points={"console_scripts": ["pyrrot=pyrrot:main"]},
        keywords="mock rest restapi service",
        package_dir={"": "src"},
        python_requires=">=3.6",
        install_requires=install_requires,
        extras_require=extras_require,
        packages=["_pyrrot", ],
        py_modules=["pyrrot"],
        zip_safe=False,
        package_data={
            '': ['**/*.html']
        }
    )


if __name__ == "__main__":
    main()
